from functools import partial

import gdsfactory as gf

xc_sin = partial(
    gf.cross_section.cross_section,
    width=1.0,
    layer=(0, 1),
    cladding_layers=[(0, 2), (0, 3)],
    cladding_offsets=[5, 10],
)

xc_sin_ec = partial(xc_sin, width=0.2)


@gf.cell
def demo_taper_cladding_offsets():
    taper_length = 10

    in_stub_length = 10
    out_stub_length = 10

    c = gf.Component()

    wg_in = c << gf.components.straight(length=in_stub_length, cross_section=xc_sin_ec)

    taper = c << gf.components.taper_cross_section_linear(
        length=taper_length, cross_section1=xc_sin_ec, cross_section2=xc_sin
    )

    wg_out = c << gf.components.straight(length=out_stub_length, cross_section=xc_sin)

    taper.connect("o1", wg_in.ports["o2"])
    wg_out.connect("o1", taper.ports["o2"])

    c.add_port("o1", port=wg_in.ports["o1"])
    c.add_port("o2", port=wg_out.ports["o2"])
    return c


def test_taper_cladding_offets():
    c = demo_taper_cladding_offsets()
    assert len(c.get_polygons()) == 9


if __name__ == "__main__":
    test_taper_cladding_offets()
    c = demo_taper_cladding_offsets()
    c.show()



from __future__ import annotations

import pathlib
from typing import Union

from gdsfactory import klive
from gdsfactory.component import Component


def show(component: Union[Component, str, pathlib.Path], **kwargs) -> None:
    """Write GDS and show Component in KLayout.

    Args:
        component: Component or GDS path.

    Keyword Args:
        gdspath: GDS file path to write to.
        gdsdir: directory for the GDS file. Defaults to /tmp/.
        unit: unit size for objects in library. 1um by default.
        precision: for object dimensions in the library (m). 1nm by default.
        timestamp: Defaults to 2019-10-25. If None uses current time.

    """
    if isinstance(component, pathlib.Path):
        component = str(component)
        return klive.show(component)
    elif isinstance(component, str):
        return klive.show(component)
    elif component is None:
        raise ValueError(
            "Component is None, make sure that your function returns the component"
        )

    elif hasattr(component, "write_gds"):
        gdspath = component.write_gds(logging=False, **kwargs)
        klive.show(gdspath)
    else:
        raise ValueError(
            f"Component is {type(component)!r}, make sure pass a Component or a path"
        )



import numpy as np

from gdsfactory.generic_tech import LAYER_STACK
from gdsfactory.simulation.fem.mode_solver import compute_cross_section_modes
from gdsfactory.technology import LayerStack


def test_compute_cross_section_mode():

    filtered_layerstack = LayerStack(
        layers={
            k: LAYER_STACK.layers[k]
            for k in (
                "core",
                "clad",
                "slab90",
                "box",
            )
        }
    )

    filtered_layerstack.layers["core"].thickness = 0.2

    resolutions = {
        "core": {"resolution": 0.02, "distance": 2},
        "clad": {"resolution": 0.2, "distance": 1},
        "box": {"resolution": 0.2, "distance": 1},
        "slab90": {"resolution": 0.05, "distance": 1},
    }
    lams, basis, xs = compute_cross_section_modes(
        cross_section="rib",
        layerstack=filtered_layerstack,
        wl=1.55,
        num_modes=4,
        order=1,
        radius=np.inf,
        resolutions=resolutions,
        overwrite=True,
    )

    assert len(lams) == 4


if __name__ == "__main__":

    test_compute_cross_section_mode()



from __future__ import annotations

import gdsfactory as gf

characters = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!#()+,-./:;<=>?@[{|}~_"


@gf.cell
def alphabet(dx=10):
    c = gf.Component()
    x = 0
    for s in characters:
        ci = gf.components.text(text=s)
        ci.name = s
        char = c << ci.flatten()
        char.x = x
        x += dx

    return c


if __name__ == "__main__":
    c = alphabet()
    c.write_gds("alphabet.gds")
    c.show()



from __future__ import annotations

import gdsfactory as gf
from gdsfactory.gdsdiff.gdsdiff import gdsdiff

if __name__ == "__main__":
    c1 = gf.components.mmi1x2(length_mmi=5)
    c2 = gf.components.mmi1x2(length_mmi=9)
    c3 = gdsdiff(c1, c2)
    gf.show(c3)
